@@ -18,6 +18,19 @@ FIREWALL_V2 = 'FIREWALL_V2'
 
 # Constants for "topics"
 FIREWALL_PLUGIN = 'q-firewall-plugin'
-L3_AGENT = 'l3_agent'
 FW_AGENT = 'firewall_agent'
 FIREWALL_RULE_LIST = 'firewall_rule_list'
+
+# V2 Constants
+DEFAULT_FWG = 'default'
+DEFAULT_FWP_INGRESS = 'default ingress'
+DEFAULT_FWP_EGRESS = 'default egress'
+
+# Firewall group events for agent-side
+DELETE_FWG = 'delete_firewall_group'
+UPDATE_FWG = 'update_firewall_group'
+CREATE_FWG = 'create_firewall_group'
+
+# Port events for L2 agent extension
+HANDLE_PORT = 'handle_port'
+DELETE_PORT = 'delete_port'
