@@ -15,13 +15,12 @@
 
 from neutron.agent.linux import iptables_manager
 from neutron.common import utils
+from neutron_lib import constants
 from neutron_lib.exceptions import firewall_v1 as f_exc
-from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import excutils
 
-from neutron_fwaas._i18n import _LE
 from neutron_fwaas.common import fwaas_constants as f_const
+from neutron_fwaas.services.firewall.drivers import conntrack_base
 from neutron_fwaas.services.firewall.drivers import fwaas_base
 
 
@@ -32,17 +31,16 @@ FWAAS_DEFAULT_CHAIN = 'fwaas-default-policy'
 FWAAS_TO_IPTABLE_ACTION_MAP = {'allow': 'ACCEPT',
                                'deny': 'DROP',
                                'reject': 'REJECT'}
-INGRESS_DIRECTION = 'ingress'
-EGRESS_DIRECTION = 'egress'
-CHAIN_NAME_PREFIX = {INGRESS_DIRECTION: 'i',
-                     EGRESS_DIRECTION: 'o'}
+
+CHAIN_NAME_PREFIX = {constants.INGRESS_DIRECTION: 'i',
+                     constants.EGRESS_DIRECTION: 'o'}
 
 """ Firewall rules are applied on internal-interfaces of Neutron router.
     The packets ingressing tenant's network will be on the output
     direction on internal-interfaces.
 """
-IPTABLES_DIR = {INGRESS_DIRECTION: '-o',
-                EGRESS_DIRECTION: '-i'}
+IPTABLES_DIR = {constants.INGRESS_DIRECTION: '-o',
+                constants.EGRESS_DIRECTION: '-i'}
 IPV4 = 'ipv4'
 IPV6 = 'ipv6'
 IP_VER_TAG = {IPV4: 'v4',
@@ -59,27 +57,7 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
     def __init__(self):
         LOG.debug("Initializing fwaas iptables driver")
         self.pre_firewall = None
-        conntrack_cls = self._load_firewall_extension_driver(
-                'neutron_fwaas.services.firewall.drivers.linux',
-                cfg.CONF.fwaas.conntrack_driver)
-        self.conntrack = conntrack_cls()
-        self.conntrack.initialize()
-
-    @staticmethod
-    def _load_firewall_extension_driver(namespace, driver):
-        """Loads driver using alias or class name
-        :param namespace: namespace where alias is defined
-        :param driver: driver alias or class name
-        :returns driver that is loaded
-        :raises ImportError if fails to load driver
-        """
-
-        try:
-            return utils.load_class_by_alias_or_classname(namespace,
-                    driver)
-        except ImportError:
-            with excutils.save_and_reraise_exception():
-                LOG.error(_LE("Driver '%s' not found."), driver)
+        self.conntrack = conntrack_base.load_and_init_conntrack_driver()
 
     def create_firewall(self, agent_mode, apply_list, firewall):
         LOG.debug('Creating firewall %(fw_id)s for tenant %(tid)s',
@@ -94,7 +72,7 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
                 self.apply_default_policy(agent_mode, apply_list, firewall)
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
-            LOG.exception(_LE("Failed to create firewall: %s"), firewall['id'])
+            LOG.exception("Failed to create firewall: %s", firewall['id'])
             raise f_exc.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def _get_ipt_mgrs_with_if_prefix(self, agent_mode, router_info):
@@ -116,7 +94,7 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
             if router_info.snat_iptables_manager:
                 ipt_mgrs.append({'ipt': router_info.snat_iptables_manager,
                                  'if_prefix': SNAT_INT_DEV_PREFIX})
-        if router_info.rtr_fip_connect:
+        if router_info.dist_fip_count:
             # handle the fip case on n/w or compute node.
             ipt_mgrs.append({'ipt': router_info.iptables_manager,
                              'if_prefix': ROUTER_2_FIP_DEV_PREFIX})
@@ -139,7 +117,7 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
             self.pre_firewall = None
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
-            LOG.exception(_LE("Failed to delete firewall: %s"), fwid)
+            LOG.exception("Failed to delete firewall: %s", fwid)
             raise f_exc.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def update_firewall(self, agent_mode, apply_list, firewall):
@@ -159,7 +137,7 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
             self.pre_firewall = dict(firewall)
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
-            LOG.exception(_LE("Failed to update firewall: %s"), firewall['id'])
+            LOG.exception("Failed to update firewall: %s", firewall['id'])
             raise f_exc.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def apply_default_policy(self, agent_mode, apply_list, firewall):
@@ -185,7 +163,7 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
             LOG.exception(
-                _LE("Failed to apply default policy on firewall: %s"), fwid)
+                "Failed to apply default policy on firewall: %s", fwid)
             raise f_exc.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def _setup_firewall(self, agent_mode, apply_list, firewall):
@@ -228,8 +206,10 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
                 table = ipt_mgr.ipv4['filter']
             else:
                 table = ipt_mgr.ipv6['filter']
-            ichain_name = self._get_chain_name(fwid, ver, INGRESS_DIRECTION)
-            ochain_name = self._get_chain_name(fwid, ver, EGRESS_DIRECTION)
+            ichain_name = self._get_chain_name(
+                fwid, ver, constants.INGRESS_DIRECTION)
+            ochain_name = self._get_chain_name(
+                fwid, ver, constants.EGRESS_DIRECTION)
             for name in [ichain_name, ochain_name]:
                 table.add_chain(name)
                 table.add_rule(name, invalid_rule)
@@ -245,8 +225,10 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
             else:
                 ver = IPV6
                 table = ipt_mgr.ipv6['filter']
-            ichain_name = self._get_chain_name(fwid, ver, INGRESS_DIRECTION)
-            ochain_name = self._get_chain_name(fwid, ver, EGRESS_DIRECTION)
+            ichain_name = self._get_chain_name(
+                fwid, ver, constants.INGRESS_DIRECTION)
+            ochain_name = self._get_chain_name(
+                fwid, ver, constants.EGRESS_DIRECTION)
             table.add_rule(ichain_name, iptbl_rule)
             table.add_rule(ochain_name, iptbl_rule)
         self._enable_policy_chain(fwid, ipt_if_prefix)
@@ -312,7 +294,8 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
     def _remove_chains(self, fwid, ipt_mgr):
         """Remove fwaas policy chain."""
         for ver in [IPV4, IPV6]:
-            for direction in [INGRESS_DIRECTION, EGRESS_DIRECTION]:
+            for direction in [constants.INGRESS_DIRECTION,
+                              constants.EGRESS_DIRECTION]:
                 chain_name = self._get_chain_name(fwid, ver, direction)
                 self._remove_chain_by_name(ver, chain_name, ipt_mgr)
 
@@ -343,7 +326,8 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
 
         for (ver, tbl) in [(IPV4, ipt_mgr.ipv4['filter']),
                            (IPV6, ipt_mgr.ipv6['filter'])]:
-            for direction in [INGRESS_DIRECTION, EGRESS_DIRECTION]:
+            for direction in [constants.INGRESS_DIRECTION,
+                              constants.EGRESS_DIRECTION]:
                 chain_name = self._get_chain_name(fwid, ver, direction)
                 chain_name = iptables_manager.get_chain_name(chain_name)
                 if chain_name in tbl.chains:
@@ -379,9 +363,9 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
 
         # iptables adds '-m protocol' when any source
         # or destination port number is specified
-        if (rule.get('source_port') is not None or
-            rule.get('destination_port') is not None):
-            args += self._match_arg(rule.get('protocol'))
+        if not((rule.get('source_port') is None)
+           and (rule.get('destination_port') is None)):
+                args += self._match_arg(rule.get('protocol'))
 
         args += self._port_arg('sport',
                                rule.get('protocol'),
@@ -430,7 +414,8 @@ class IptablesFwaasDriver(fwaas_base.FwaasDriverBase):
         return args
 
     def _port_arg(self, direction, protocol, port):
-        if protocol not in ['udp', 'tcp'] or port is None:
+        if (protocol not in ['udp', 'tcp']
+            or port is None):
             return []
 
         args = ['--%s' % direction, '%s' % port]
