@@ -14,12 +14,12 @@
 #    under the License.
 
 from neutron.agent.linux import iptables_manager
-from neutron.agent.linux import utils as linux_utils
 from neutron.common import utils
+from neutron_lib import constants
 from neutron_lib.exceptions import firewall_v2 as fw_ext
 from oslo_log import log as logging
 
-from neutron_fwaas._i18n import _LE
+from neutron_fwaas.services.firewall.drivers import conntrack_base
 from neutron_fwaas.services.firewall.drivers import fwaas_base_v2
 
 LOG = logging.getLogger(__name__)
@@ -29,17 +29,16 @@ FWAAS_DEFAULT_CHAIN = 'fwaas-default-policy'
 FWAAS_TO_IPTABLE_ACTION_MAP = {'allow': 'ACCEPT',
                                'deny': 'DROP',
                                'reject': 'REJECT'}
-INGRESS_DIRECTION = 'ingress'
-EGRESS_DIRECTION = 'egress'
-CHAIN_NAME_PREFIX = {INGRESS_DIRECTION: 'i',
-                     EGRESS_DIRECTION: 'o'}
+
+CHAIN_NAME_PREFIX = {constants.INGRESS_DIRECTION: 'i',
+                     constants.EGRESS_DIRECTION: 'o'}
 
 """ Firewall rules are applied on internal-interfaces of Neutron router.
     The packets ingressing tenant's network will be on the output
     direction on internal-interfaces.
 """
-IPTABLES_DIR = {INGRESS_DIRECTION: '-o',
-                EGRESS_DIRECTION: '-i'}
+IPTABLES_DIR = {constants.INGRESS_DIRECTION: '-o',
+                constants.EGRESS_DIRECTION: '-i'}
 IPV4 = 'ipv4'
 IPV6 = 'ipv6'
 IP_VER_TAG = {IPV4: 'v4',
@@ -58,9 +57,7 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
     def __init__(self):
         LOG.debug("Initializing fwaas iptables driver")
         self.pre_firewall = None
-
-    def initialize(self):
-        pass
+        self.conntrack = conntrack_base.load_and_init_conntrack_driver()
 
     def _get_intf_name(self, if_prefix, port_id):
         _name = "%s%s" % (if_prefix, port_id)
@@ -79,7 +76,7 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
                 self.apply_default_policy(agent_mode, apply_list, firewall)
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
-            LOG.exception(_LE("Failed to create firewall: %s"), firewall['id'])
+            LOG.exception("Failed to create firewall: %s", firewall['id'])
             raise fw_ext.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def _get_ipt_mgrs_with_if_prefix(self, agent_mode, ri):
@@ -101,7 +98,7 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
             if ri.snat_iptables_manager:
                 ipt_mgrs.append({'ipt': ri.snat_iptables_manager,
                                  'if_prefix': SNAT_INT_DEV_PREFIX})
-        if ri.rtr_fip_connect:
+        if ri.dist_fip_count:
             # handle the fip case on n/w or compute node.
             ipt_mgrs.append({'ipt': ri.iptables_manager,
                              'if_prefix': ROUTER_2_FIP_DEV_PREFIX})
@@ -124,7 +121,7 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
             self.pre_firewall = None
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
-            LOG.exception(_LE("Failed to delete firewall: %s"), fwid)
+            LOG.exception("Failed to delete firewall: %s", fwid)
             raise fw_ext.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def update_firewall_group(self, agent_mode, apply_list, firewall):
@@ -144,7 +141,7 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
             self.pre_firewall = dict(firewall)
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
-            LOG.exception(_LE("Failed to update firewall: %s"), firewall['id'])
+            LOG.exception("Failed to update firewall: %s", firewall['id'])
             raise fw_ext.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def apply_default_policy(self, agent_mode, apply_list, firewall):
@@ -171,7 +168,7 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
         except (LookupError, RuntimeError):
             # catch known library exceptions and raise Fwaas generic exception
             LOG.exception(
-                _LE("Failed to apply default policy on firewall: %s"), fwid)
+                "Failed to apply default policy on firewall: %s", fwid)
             raise fw_ext.FirewallInternalDriverError(driver=FWAAS_DRIVER_NAME)
 
     def _setup_firewall(self, agent_mode, apply_list, firewall):
@@ -215,8 +212,10 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
                 table = ipt_mgr.ipv4['filter']
             else:
                 table = ipt_mgr.ipv6['filter']
-            ichain_name = self._get_chain_name(fwid, ver, INGRESS_DIRECTION)
-            ochain_name = self._get_chain_name(fwid, ver, EGRESS_DIRECTION)
+            ichain_name = self._get_chain_name(
+                fwid, ver, constants.INGRESS_DIRECTION)
+            ochain_name = self._get_chain_name(
+                fwid, ver, constants.EGRESS_DIRECTION)
             for name in [ichain_name, ochain_name]:
                 table.add_chain(name)
                 table.add_rule(name, invalid_rule)
@@ -232,7 +231,8 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
             else:
                 ver = IPV6
                 table = ipt_mgr.ipv6['filter']
-            ichain_name = self._get_chain_name(fwid, ver, INGRESS_DIRECTION)
+            ichain_name = self._get_chain_name(
+                fwid, ver, constants.INGRESS_DIRECTION)
             table.add_rule(ichain_name, iptbl_rule)
 
         for rule in egress_rule_list:
@@ -245,7 +245,8 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
             else:
                 ver = IPV6
                 table = ipt_mgr.ipv6['filter']
-            ochain_name = self._get_chain_name(fwid, ver, EGRESS_DIRECTION)
+            ochain_name = self._get_chain_name(
+                fwid, ver, constants.EGRESS_DIRECTION)
             table.add_rule(ochain_name, iptbl_rule)
 
         self._enable_policy_chain(fwid, ipt_if_prefix, router_fw_ports)
@@ -279,26 +280,6 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
     def _find_new_rules(self, pre_firewall, firewall):
         return self._find_removed_rules(firewall, pre_firewall)
 
-    def _get_conntrack_cmd_from_rule(self, ipt_mgr, rule=None):
-        prefixcmd = ['ip', 'netns', 'exec'] + [ipt_mgr.namespace]
-        cmd = ['conntrack', '-D']
-        if rule:
-            conntrack_filter = self._get_conntrack_filter_from_rule(rule)
-            exec_cmd = prefixcmd + cmd + conntrack_filter
-        else:
-            exec_cmd = prefixcmd + cmd
-        return exec_cmd
-
-    def _remove_conntrack_by_cmd(self, cmd):
-        if cmd:
-            try:
-                linux_utils.execute(cmd, run_as_root=True,
-                                    check_exit_code=True,
-                                    extra_ok_codes=[1])
-            except RuntimeError:
-                LOG.exception(
-                        _LE("Failed execute conntrack command %s"), str(cmd))
-
     def _remove_conntrack_new_firewall(self, agent_mode, apply_list, firewall):
         """Remove conntrack when create new firewall"""
         routers_list = list(set([apply_info[0] for apply_info in apply_list]))
@@ -307,8 +288,7 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
                 agent_mode, ri)
             for ipt_if_prefix in ipt_if_prefix_list:
                 ipt_mgr = ipt_if_prefix['ipt']
-                cmd = self._get_conntrack_cmd_from_rule(ipt_mgr)
-                self._remove_conntrack_by_cmd(cmd)
+                self.conntrack.flush_entries(ipt_mgr.namespace)
 
     def _remove_conntrack_updated_firewall(self, agent_mode,
                                            apply_list, pre_firewall, firewall):
@@ -324,27 +304,8 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
                 i_rules = self._find_new_rules(pre_firewall, firewall)
                 r_rules = self._find_removed_rules(pre_firewall, firewall)
                 removed_conntrack_rules_list = ch_rules + i_rules + r_rules
-                for rule in removed_conntrack_rules_list:
-                    cmd = self._get_conntrack_cmd_from_rule(ipt_mgr, rule)
-                    self._remove_conntrack_by_cmd(cmd)
-
-    def _get_conntrack_filter_from_rule(self, rule):
-        """Get conntrack filter from rule.
-        The key for get conntrack filter is protocol, destination_port
-        and source_port. If we want to take more keys, add to the list.
-        """
-        conntrack_filter = []
-        keys = [['-p', 'protocol'], ['-f', 'ip_version'],
-                ['--dport', 'destination_port'], ['--sport', 'source_port']]
-        for key in keys:
-            if rule.get(key[1]):
-                if key[1] == 'ip_version':
-                    conntrack_filter.append(key[0])
-                    conntrack_filter.append('ipv' + str(rule.get(key[1])))
-                else:
-                    conntrack_filter.append(key[0])
-                    conntrack_filter.append(rule.get(key[1]))
-        return conntrack_filter
+                self.conntrack.delete_entries(removed_conntrack_rules_list,
+                                              ipt_mgr.namespace)
 
     def _remove_default_chains(self, nsid):
         """Remove fwaas default policy chain."""
@@ -354,7 +315,8 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
     def _remove_chains(self, fwid, ipt_mgr):
         """Remove fwaas policy chain."""
         for ver in [IPV4, IPV6]:
-            for direction in [INGRESS_DIRECTION, EGRESS_DIRECTION]:
+            for direction in [constants.INGRESS_DIRECTION,
+                              constants.EGRESS_DIRECTION]:
                 chain_name = self._get_chain_name(fwid, ver, direction)
                 self._remove_chain_by_name(ver, chain_name, ipt_mgr)
 
@@ -385,7 +347,8 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
 
         for (ver, tbl) in [(IPV4, ipt_mgr.ipv4['filter']),
                            (IPV6, ipt_mgr.ipv6['filter'])]:
-            for direction in [INGRESS_DIRECTION, EGRESS_DIRECTION]:
+            for direction in [constants.INGRESS_DIRECTION,
+                              constants.EGRESS_DIRECTION]:
                 chain_name = self._get_chain_name(fwid, ver, direction)
                 chain_name = iptables_manager.get_chain_name(chain_name)
                 if chain_name in tbl.chains:
@@ -431,8 +394,8 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
 
         # iptables adds '-m protocol' when any source
         # or destination port number is specified
-        if (rule.get('source_port') is not None or
-            rule.get('destination_port') is not None):
+        if not((rule.get('source_port') is None)
+           and (rule.get('destination_port') is None)):
                 args += self._match_arg(rule.get('protocol'))
 
         args += self._port_arg('sport',
@@ -482,7 +445,8 @@ class IptablesFwaasDriver(fwaas_base_v2.FwaasDriverBase):
         return args
 
     def _port_arg(self, direction, protocol, port):
-        if protocol not in ['udp', 'tcp'] or port is None:
+        if (protocol not in ['udp', 'tcp']
+            or port is None):
             return []
 
         args = ['--%s' % direction, '%s' % port]
