@@ -16,6 +16,10 @@ from neutron.common import rpc as n_rpc
 from neutron.db import servicetype_db as st_db
 from neutron.services import provider_configuration as provider_conf
 from neutron_lib.api.definitions import firewall_v2
+from neutron_lib.api.definitions import portbindings as pb_def
+from neutron_lib.callbacks import events
+from neutron_lib.callbacks import registry
+from neutron_lib.callbacks import resources
 from neutron_lib import constants as nl_constants
 from neutron_lib import context as neutron_context
 from neutron_lib.exceptions import firewall_v2 as f_exc
@@ -25,11 +29,10 @@ from oslo_config import cfg
 from oslo_log import log as logging
 import oslo_messaging
 
-from neutron_fwaas._i18n import _LI
+from neutron_fwaas.common import exceptions
 from neutron_fwaas.common import fwaas_constants
 from neutron_fwaas.db.firewall.v2 import firewall_db_v2
 
-
 LOG = logging.getLogger(__name__)
 
 
@@ -112,7 +115,7 @@ class FirewallCallbacks(object):
                     fwg_db.update({"status": nl_constants.ERROR})
                     return False
         except f_exc.FirewallGroupNotFound:
-            LOG.info(_LI('Firewall group %s already deleted'), fwg_id)
+            LOG.info('Firewall group %s already deleted', fwg_id)
             return True
 
     def get_firewall_groups_for_project(self, context, **kwargs):
@@ -143,7 +146,15 @@ class FirewallCallbacks(object):
         fwg_project_list = list(set(fwg['tenant_id'] for fwg in fwg_list))
         return fwg_project_list
 
+    def get_firewall_group_for_port(self, context, **kwargs):
+        """Get firewall_group is associated  with a port."""
+        LOG.debug("get_firewall_group_for_port() called")
+        ctx = context.elevated()
+        return self.plugin.get_firewall_group_for_port(
+            ctx, kwargs.get('port_id'))
 
+
+@registry.has_registry_receivers
 class FirewallPluginV2(
     firewall_db_v2.Firewall_db_mixin_v2):
     """Implementation of the Neutron Firewall Service Plugin.
@@ -191,6 +202,8 @@ class FirewallPluginV2(
         fwg_with_rules['add-port-ids'] = self._get_ports_in_firewall_group(
                 context, fwg_id)
         fwg_with_rules['del-port-ids'] = []
+        fwg_with_rules['port_details'] = self._get_fwg_port_details(
+            context, fwg_with_rules['add-port-ids'])
         self.agent_rpc.update_firewall_group(context, fwg_with_rules)
 
     def _rpc_update_firewall_policy(self, context, firewall_policy_id):
@@ -212,8 +225,8 @@ class FirewallPluginV2(
     def _ensure_update_firewall_policy(self, context, firewall_policy_id):
         firewall_policy = self.get_firewall_policy(context, firewall_policy_id)
         if firewall_policy:
-            ing_fwg_ids, eg_fwg_ids = self._get_fwgs_with_policy(context,
-                firewall_policy_id)
+            ing_fwg_ids, eg_fwg_ids = self._get_fwgs_with_policy(
+                context, firewall_policy_id)
             for fwg_id in list(set(ing_fwg_ids + eg_fwg_ids)):
                 self._ensure_update_firewall_group(context, fwg_id)
 
@@ -223,16 +236,46 @@ class FirewallPluginV2(
             self._ensure_update_firewall_policy(context, fwp_id)
 
     def _validate_ports_for_firewall_group(self, context, tenant_id,
-        fwg_ports):
+                                           fwg_ports):
         # TODO(sridar): elevated context and do we want to use public ?
         for port_id in fwg_ports:
             port_db = self._core_plugin._get_port(context, port_id)
-            if port_db['device_owner'] != "network:router_interface":
-                raise f_exc.FirewallGroupPortInvalid(port_id=port_id)
             if port_db['tenant_id'] != tenant_id:
                 raise f_exc.FirewallGroupPortInvalidProject(
                     port_id=port_id, project_id=port_db['tenant_id'])
-        return
+            device_owner = port_db.get('device_owner', '')
+            if (device_owner not in [nl_constants.DEVICE_OWNER_ROUTER_INTF]
+                and not device_owner.startswith(
+                    nl_constants.DEVICE_OWNER_COMPUTE_PREFIX)):
+                raise f_exc.FirewallGroupPortInvalid(port_id=port_id)
+            if (device_owner.startswith(
+                    nl_constants.DEVICE_OWNER_COMPUTE_PREFIX) and not
+                self._is_supported_by_fw_l2_driver(context, port_id)):
+                raise exceptions.FirewallGroupPortNotSupported(port_id=port_id)
+
+    def _is_supported_by_fw_l2_driver(self, context, port_id):
+        """Whether this port is supported by firewall l2 driver"""
+
+        # Re-fetch to get up-to-date data from db
+        port = self._core_plugin.get_port(context, id=port_id)
+
+        # Skip port binding is unbound or failed
+        if port[pb_def.VIF_TYPE] in [pb_def.VIF_TYPE_UNBOUND,
+                                     pb_def.VIF_TYPE_BINDING_FAILED]:
+            return False
+
+        if not port['port_security_enabled']:
+            return True
+
+        if port[pb_def.VIF_TYPE] == pb_def.VIF_TYPE_OVS:
+            # TODO(annp): remove these lines after we fully support for hybrid
+            # port
+            if not port[pb_def.VIF_DETAILS][pb_def.OVS_HYBRID_PLUG]:
+                return True
+            LOG.warning("Doesn't support hybrid port at the moment")
+        else:
+            LOG.warning("Doesn't support vif type %s", port[pb_def.VIF_TYPE])
+        return False
 
     def _check_no_need_pending(self, context, fwg_id, fwg_body):
         fwg_db = self._get_firewall_group(context, fwg_id)
@@ -246,10 +289,65 @@ class FirewallPluginV2(
             return True
         return False
 
+    def _get_fwg_port_details(self, context, fwg_ports):
+        """Returns a dictionary list of port details. """
+        port_details = {}
+        for port_id in fwg_ports:
+            port_db = self._core_plugin.get_port(context, port_id)
+            # Add more parameters below based on requirement.
+            device_owner = port_db['device_owner']
+            port_details[port_id] = {
+                'device_owner': device_owner,
+                'device': port_db['id'],
+                'network_id': port_db['network_id'],
+                'fixed_ips': port_db['fixed_ips'],
+                'allowed_address_pairs':
+                    port_db.get('allowed_address_pairs', []),
+                'port_security_enabled':
+                    port_db.get('port_security_enabled', True),
+                'id': port_db['id']
+            }
+            if device_owner.startswith(
+                nl_constants.DEVICE_OWNER_COMPUTE_PREFIX):
+                port_details[port_id].update(
+                    {'host': port_db[pb_def.HOST_ID]})
+        return port_details
+
+    def get_project_id_from_port_id(self, context, port_id):
+        """Returns an ID of project for specified port_id. """
+        return self._core_plugin.get_port(context, port_id)['project_id']
+
+    @registry.receives(resources.PORT, [events.AFTER_UPDATE])
+    def handle_update_port(self, resource, event, trigger, **kwargs):
+
+        updated_port = kwargs['port']
+        if not updated_port['device_owner'].startswith(
+                nl_constants.DEVICE_OWNER_COMPUTE_PREFIX):
+            return
+
+        if (kwargs.get('original_port')[pb_def.VIF_TYPE] !=
+                pb_def.VIF_TYPE_UNBOUND):
+            # Checking newly vm port binding allows us to avoid call to DB
+            # when a port update_event like restart, setting name, etc...
+            # Moreover, that will help us in case of tenant admin wants to
+            # only attach security group to vm port.
+            return
+
+        context = kwargs['context']
+        port_id = updated_port['id']
+        # Check port is supported by firewall l2 driver or not
+        if not self._is_supported_by_fw_l2_driver(context, port_id):
+            return
+
+        project_id = updated_port['project_id']
+        LOG.debug("Try to associate port %s at %s", port_id, project_id)
+        self.set_port_for_default_firewall_group(context, port_id, project_id)
+
     def create_firewall_group(self, context, firewall_group):
         LOG.debug("create_firewall_group() called")
         fwgrp = firewall_group['firewall_group']
         fwg_ports = fwgrp['ports']
+
         if not fwg_ports:
             # no messaging to agent needed, and fw needs to go
             # to INACTIVE(no associated ports) state.
@@ -261,9 +359,7 @@ class FirewallPluginV2(
         else:
             # Validate ports
             self._validate_ports_for_firewall_group(context,
-                firewall_group['firewall_group']['tenant_id'],
-                fwg_ports)
-            self._validate_if_firewall_group_on_ports(context, fwg_ports)
+                firewall_group['firewall_group']['tenant_id'], fwg_ports)
 
             if (not fwgrp['ingress_firewall_policy_id'] and
                 not fwgrp['egress_firewall_policy_id']):
@@ -281,7 +377,9 @@ class FirewallPluginV2(
             self._make_firewall_group_dict_with_rules(context, fwg['id']))
 
         fwg_with_rules['add-port-ids'] = fwg_ports
-        fwg_with_rules['del-ports-ids'] = []
+        fwg_with_rules['del-port-ids'] = []
+        fwg_with_rules['port_details'] = self._get_fwg_port_details(
+            context, fwg_ports)
 
         self.agent_rpc.create_firewall_group(context, fwg_with_rules)
 
@@ -296,16 +394,10 @@ class FirewallPluginV2(
         fwg_current_ports = fwg_new_ports = self._get_ports_in_firewall_group(
             context, id)
         if 'ports' in firewall_group['firewall_group']:
-            fwg_ports = firewall_group['firewall_group']['ports']
-            if fwg_ports == []:
-                # This indicates that user is indicating no ports.
-                fwg_new_ports = []
-            else:
+            fwg_new_ports = firewall_group['firewall_group']['ports']
+            if len(fwg_new_ports) > 0:
                 self._validate_ports_for_firewall_group(
-                    context, context.tenant_id, fwg_ports)
-                self._validate_if_firewall_group_on_ports(
-                    context, fwg_ports, id)
-                fwg_new_ports = fwg_ports
+                    context, context.project_id, fwg_new_ports)
 
         if ((not fwg_new_ports and not fwg_current_ports) or
             self._check_no_need_pending(context,
@@ -345,6 +437,10 @@ class FirewallPluginV2(
             fwg_with_rules['add-port-ids'],
             fwg_with_rules['del-port-ids'])
 
+        fwg_with_rules['port_details'] = self._get_fwg_port_details(
+            context, fwg_with_rules['del-port-ids'])
+        fwg_with_rules['port_details'].update(self._get_fwg_port_details(
+            context, fwg_with_rules['add-port-ids']))
         self.agent_rpc.update_firewall_group(context, fwg_with_rules)
 
         return fwg
@@ -354,22 +450,30 @@ class FirewallPluginV2(
 
     def delete_firewall_group(self, context, id):
         LOG.debug("delete_firewall_group() called on firewall_group %s", id)
-        fw_with_rules = (
+
+        fwg_db = self._get_firewall_group(context, id)
+
+        if fwg_db['status'] == nl_constants.ACTIVE:
+            raise f_exc.FirewallGroupInUse(firewall_id=id)
+
+        fwg_with_rules = (
             self._make_firewall_group_dict_with_rules(context, id))
-        fw_with_rules['del-port-ids'] = self._get_ports_in_firewall_group(
+        fwg_with_rules['del-port-ids'] = self._get_ports_in_firewall_group(
             context, id)
-        fw_with_rules['add-port-ids'] = []
-        if not fw_with_rules['del-port-ids']:
+        fwg_with_rules['add-port-ids'] = []
+        if not fwg_with_rules['del-port-ids']:
             # no ports, no need to talk to the agent
             self.delete_db_firewall_group_object(context, id)
         else:
-            status = {"firewall_group":
-                     {"status": nl_constants.PENDING_DELETE}}
+            status = {"firewall_group": {"status":
+                                         nl_constants.PENDING_DELETE}}
             super(FirewallPluginV2, self).update_firewall_group(
                 context, id, status)
-            # Reflect state change in fw_with_rules
-            fw_with_rules['status'] = status['firewall_group']['status']
-            self.agent_rpc.delete_firewall_group(context, fw_with_rules)
+            # Reflect state change in fwg_with_rules
+            fwg_with_rules['status'] = status['firewall_group']['status']
+            fwg_with_rules['port_details'] = self._get_fwg_port_details(
+                context, fwg_with_rules['del-port-ids'])
+            self.agent_rpc.delete_firewall_group(context, fwg_with_rules)
 
     def update_firewall_policy(self, context, id, firewall_policy):
         LOG.debug("update_firewall_policy() called")
@@ -389,18 +493,16 @@ class FirewallPluginV2(
             self._rpc_update_firewall_policy(context, fwp_id)
         return fwr
 
-    def insert_rule(self, context, policy_id, rule_info):
-        LOG.debug("insert_rule() called for policy %s ", policy_id)
-        self._ensure_update_firewall_policy(context, policy_id)
-        fwp = super(FirewallPluginV2, self).insert_rule(context, policy_id,
-                                                        rule_info)
-        self._rpc_update_firewall_policy(context, policy_id)
+    def insert_rule(self, context, id, rule_info):
+        LOG.debug("insert_rule() called")
+        self._ensure_update_firewall_policy(context, id)
+        fwp = super(FirewallPluginV2, self).insert_rule(context, id, rule_info)
+        self._rpc_update_firewall_policy(context, id)
         return fwp
 
-    def remove_rule(self, context, policy_id, rule_info):
-        LOG.debug("remove_rule() called for policy %s ", policy_id)
-        self._ensure_update_firewall_policy(context, policy_id)
-        fwp = super(FirewallPluginV2, self).remove_rule(context, policy_id,
-                                                        rule_info)
-        self._rpc_update_firewall_policy(context, policy_id)
+    def remove_rule(self, context, id, rule_info):
+        LOG.debug("remove_rule() called")
+        self._ensure_update_firewall_policy(context, id)
+        fwp = super(FirewallPluginV2, self).remove_rule(context, id, rule_info)
+        self._rpc_update_firewall_policy(context, id)
         return fwp
