@@ -22,6 +22,7 @@ from tempest.lib import decorators
 from tempest.lib import exceptions as lib_exc
 from tempest import test
 
+from neutron_fwaas.common import fwaas_constants
 from neutron_fwaas.tests.tempest_plugin.tests.api import v2_base
 
 CONF = config.CONF
@@ -210,7 +211,11 @@ class FWaaSv2ExtensionTestJSON(v2_base.BaseFWaaSTest):
         fw_rule = self.firewall_rules_client.show_firewall_rule(
             self.fw_rule_1['id'])
         for key, value in six.iteritems(fw_rule['firewall_rule']):
-            self.assertEqual(self.fw_rule_1[key], value)
+            if key != 'firewall_policy_id':
+                self.assertEqual(self.fw_rule_1[key], value)
+            # This check is placed because we cannot modify policy during
+            # Create/Update of Firewall Rule but we can see the association
+            # of a Firewall Rule with the policies it belongs to.
 
     @decorators.idempotent_id('f6b83902-746f-4e74-9403-2ec9899583a3')
     def test_list_firewall_policies(self):
@@ -267,7 +272,6 @@ class FWaaSv2ExtensionTestJSON(v2_base.BaseFWaaSTest):
             ports=[intf_1['port_id'], intf_2['port_id']])
         created_firewall_group = body['firewall_group']
         fwg_id = created_firewall_group['id']
-        self.addCleanup(self._try_delete_firewall_group, fwg_id)
 
         # Wait for the firewall resource to become ready
         self._wait_until_ready(fwg_id)
@@ -294,6 +298,8 @@ class FWaaSv2ExtensionTestJSON(v2_base.BaseFWaaSTest):
                         m['ingress_firewall_policy_id'],
                         m['egress_firewall_policy_id']) for m in fwgs])
 
+        # Disassociate all port with this firewall group
+        self.firewall_groups_client.update_firewall_group(fwg_id, ports=[])
         # Delete firewall_group
         self.firewall_groups_client.delete_firewall_group(fwg_id)
 
@@ -332,3 +338,19 @@ class FWaaSv2ExtensionTestJSON(v2_base.BaseFWaaSTest):
 
         # Delete firewall_group
         self.firewall_groups_client.delete_firewall_group(fwg_id)
+
+    @decorators.idempotent_id('a1f524d8-5336-4769-aa7b-0830bb4df6c8')
+    def test_error_on_create_firewall_group_name_default(self):
+        try:
+            # Create firewall_group with name == reserved default fwg
+            self.firewall_groups_client.create_firewall_group(
+                name=fwaas_constants.DEFAULT_FWG)
+        except lib_exc.Conflict:
+            pass
+
+    @decorators.idempotent_id('fd24db16-c8cb-4cb4-ba60-b0cd18a66b83')
+    def test_default_fwg_created_on_list_firewall_groups(self):
+        fw_groups = self.firewall_groups_client.list_firewall_groups()
+        fw_groups = fw_groups['firewall_groups']
+        self.assertIn(fwaas_constants.DEFAULT_FWG,
+                      [g['name'] for g in fw_groups])
