@@ -14,10 +14,11 @@
 #    under the License.
 
 import copy
-import mock
 
+import mock
 from neutron.tests import base
 from neutron.tests.unit.api.v2 import test_base as test_api_v2
+
 import neutron_fwaas.services.firewall.drivers.linux.iptables_fwaas as fwaas
 
 
@@ -142,8 +143,8 @@ class IptablesFwaasTestCase(base.BaseTestCase):
             distributed_mode = 'legacy'
         func(distributed_mode, apply_list, firewall)
         invalid_rule = '-m state --state INVALID -j DROP'
-        est_rule = '-m state --state ESTABLISHED,RELATED -j ACCEPT'
-        rule1 = '-p tcp -m tcp --dport 80 -s 10.24.4.2 -j ACCEPT'
+        est_rule = '-m state --state RELATED,ESTABLISHED -j ACCEPT'
+        rule1 = '-p tcp -s 10.24.4.2/32 -m tcp --dport 80 -j ACCEPT'
         rule2 = '-p tcp -m tcp --dport 22 -j DROP'
         rule3 = '-p tcp -m tcp --dport 23 -j REJECT'
         ingress_chain = 'iv4%s' % firewall['id']
@@ -189,7 +190,7 @@ class IptablesFwaasTestCase(base.BaseTestCase):
         firewall = self._fake_firewall_no_rule()
         self.firewall.create_firewall('legacy', apply_list, firewall)
         invalid_rule = '-m state --state INVALID -j DROP'
-        est_rule = '-m state --state ESTABLISHED,RELATED -j ACCEPT'
+        est_rule = '-m state --state RELATED,ESTABLISHED -j ACCEPT'
         bname = fwaas.iptables_manager.binary_name
         for ip_version in (4, 6):
             ingress_chain = ('iv%s%s' % (ip_version, firewall['id']))
