@@ -14,31 +14,33 @@
 #  under the License.
 
 import mock
-import testtools
-
 from neutron.api import extensions as api_ext
-from neutron.api.v2 import attributes as attr
 from neutron.common import config
-from neutron import context
 from neutron.tests.common import helpers
 from neutron.tests import fake_notifier
 from neutron.tests.unit.extensions import test_agent
 from neutron.tests.unit.extensions import test_l3 as test_l3_plugin
+from neutron_lib.api import attributes as attr
+from neutron_lib.api.definitions import firewall as fwaas_def
+from neutron_lib.api.definitions import firewallrouterinsertion
+from neutron_lib import constants as nl_constants
+from neutron_lib import context
+from neutron_lib.exceptions import firewall_v1 as f_exc
+from neutron_lib.plugins import constants as plugin_constants
+from neutron_lib.plugins import directory
 from oslo_config import cfg
+from oslo_utils import uuidutils
 import six
-import uuid
+import testtools
 from webob import exc
 
 from neutron_fwaas.db.firewall import firewall_db as fdb
 import neutron_fwaas.extensions
 from neutron_fwaas.extensions import firewall
-from neutron_fwaas.extensions import firewallrouterinsertion
 from neutron_fwaas.services.firewall import fwaas_plugin
 from neutron_fwaas.tests import base
 from neutron_fwaas.tests.unit.db.firewall import (
     test_firewall_db as test_db_firewall)
-from neutron_lib import constants as nl_constants
-from neutron_lib.plugins import directory
 
 extensions_path = neutron_fwaas.extensions.__path__[0]
 
@@ -51,8 +53,8 @@ class FirewallTestExtensionManager(test_l3_plugin.L3TestExtensionManager):
 
     def get_resources(self):
         res = super(FirewallTestExtensionManager, self).get_resources()
-        firewall.RESOURCE_ATTRIBUTE_MAP['firewalls'].update(
-            firewallrouterinsertion.EXTENDED_ATTRIBUTES_2_0['firewalls'])
+        fwaas_def.RESOURCE_ATTRIBUTE_MAP['firewalls'].update(
+            firewallrouterinsertion.RESOURCE_ATTRIBUTE_MAP['firewalls'])
         return res + firewall.Firewall.get_resources()
 
     def get_actions(self):
@@ -78,9 +80,8 @@ class TestFirewallRouterInsertionBase(
 
         cfg.CONF.set_override('api_extensions_path', extensions_path)
         self.saved_attr_map = {}
-        for resource, attrs in six.iteritems(attr.RESOURCE_ATTRIBUTE_MAP):
+        for resource, attrs in six.iteritems(attr.RESOURCES):
             self.saved_attr_map[resource] = attrs.copy()
-        self.addCleanup(self.restore_attribute_map)
         if not fw_plugin:
             fw_plugin = FW_PLUGIN_KLASS
         service_plugins = {'l3_plugin_name': l3_plugin,
@@ -91,17 +92,18 @@ class TestFirewallRouterInsertionBase(
         super(test_db_firewall.FirewallPluginDbTestCase, self).setUp(
             plugin=plugin, service_plugins=service_plugins, ext_mgr=ext_mgr)
 
+        self.addCleanup(self.restore_attribute_map)
         self.setup_notification_driver()
 
-        self.l3_plugin = directory.get_plugin(nl_constants.L3)
+        self.l3_plugin = directory.get_plugin(plugin_constants.L3)
         self.plugin = directory.get_plugin('FIREWALL')
         self.callbacks = self.plugin.endpoints[0]
 
     def restore_attribute_map(self):
-        # Remove the csrfirewallinsertion extension
-        firewall.RESOURCE_ATTRIBUTE_MAP['firewalls'].pop('router_ids')
+        # Remove the fwaasrouterinsertion extension
+        fwaas_def.RESOURCE_ATTRIBUTE_MAP['firewalls'].pop('router_ids')
         # Restore the original RESOURCE_ATTRIBUTE_MAP
-        attr.RESOURCE_ATTRIBUTE_MAP = self.saved_attr_map
+        attr.RESOURCES = self.saved_attr_map
 
     def _create_firewall(self, fmt, name, description, firewall_policy_id=None,
                          admin_state_up=True, expected_res_status=None,
@@ -132,11 +134,6 @@ class TestFirewallRouterInsertionBase(
 
 class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
 
-    def setUp(self):
-        super(TestFirewallCallbacks,
-              self).setUp(fw_plugin=FW_PLUGIN_KLASS)
-        self.callbacks = self.plugin.endpoints[0]
-
     def test_set_firewall_status(self):
         ctx = context.get_admin_context()
         with self.firewall_policy() as fwp:
@@ -147,8 +144,7 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
             ) as fw:
                 fw_id = fw['firewall']['id']
                 res = self.callbacks.set_firewall_status(ctx, fw_id,
-                                                         nl_constants.ACTIVE,
-                                                         host='dummy')
+                                                         nl_constants.ACTIVE)
                 fw_db = self.plugin.get_firewall(ctx, fw_id)
                 self.assertEqual(nl_constants.ACTIVE, fw_db['status'])
                 self.assertTrue(res)
@@ -171,8 +167,7 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
                 fw_db['status'] = nl_constants.PENDING_DELETE
                 ctx.session.flush()
                 res = self.callbacks.set_firewall_status(ctx, fw_id,
-                                                         nl_constants.ACTIVE,
-                                                         host='dummy')
+                                                         nl_constants.ACTIVE)
                 fw_db = self.plugin.get_firewall(ctx, fw_id)
                 self.assertEqual(nl_constants.PENDING_DELETE, fw_db['status'])
                 self.assertFalse(res)
@@ -189,10 +184,9 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
                     fw_db = self.plugin._get_firewall(ctx, fw_id)
                     fw_db['status'] = nl_constants.PENDING_DELETE
                     ctx.session.flush()
-                    res = self.callbacks.firewall_deleted(ctx, fw_id,
-                                                          host='dummy')
+                    res = self.callbacks.firewall_deleted(ctx, fw_id)
                     self.assertTrue(res)
-                    self.assertRaises(firewall.FirewallNotFound,
+                    self.assertRaises(f_exc.FirewallNotFound,
                                       self.plugin.get_firewall,
                                       ctx, fw_id)
 
@@ -224,17 +218,16 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
                 with mock.patch.object(
                     self.plugin, '_get_firewall', side_effect=getdelete
                 ):
-                    observed = self.callbacks.firewall_deleted(
-                        ctx, fw_id, host='dummy')
+                    observed = self.callbacks.firewall_deleted(ctx, fw_id)
                     self.assertTrue(observed)
 
-                self.assertRaises(firewall.FirewallNotFound,
+                self.assertRaises(f_exc.FirewallNotFound,
                                   self.plugin.get_firewall,
                                   ctx, fw_id)
 
     def test_firewall_deleted_not_found(self):
         ctx = context.get_admin_context()
-        observed = self.callbacks.firewall_deleted(ctx, 'notfound', host='hh')
+        observed = self.callbacks.firewall_deleted(ctx, 'notfound')
         self.assertTrue(observed)
 
     def test_firewall_deleted_error(self):
@@ -246,8 +239,7 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
                 admin_state_up=test_db_firewall.ADMIN_STATE_UP,
             ) as fw:
                 fw_id = fw['firewall']['id']
-                res = self.callbacks.firewall_deleted(ctx, fw_id,
-                                                      host='dummy')
+                res = self.callbacks.firewall_deleted(ctx, fw_id)
                 self.assertFalse(res)
                 fw_db = self.plugin._get_firewall(ctx, fw_id)
                 self.assertEqual(nl_constants.ERROR, fw_db['status'])
@@ -274,8 +266,7 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
                         tenant_id=tenant_id,
                         admin_state_up=test_db_firewall.ADMIN_STATE_UP) as fw:
                     fw_id = fw['firewall']['id']
-                    res = self.callbacks.get_firewalls_for_tenant(ctx,
-                                                                  host='dummy')
+                    res = self.callbacks.get_firewalls_for_tenant(ctx)
                     fw_rules = (
                         self.plugin._make_firewall_dict_with_rules(ctx,
                                                                    fw_id)
@@ -286,25 +277,6 @@ class TestFirewallCallbacks(TestFirewallRouterInsertionBase):
                     self._compare_firewall_rule_lists(
                         fwp_id, fr, res[0]['firewall_rule_list'])
 
-    def test_get_firewall_for_tenant_without_rules(self):
-        tenant_id = 'test-tenant'
-        ctx = context.Context('', tenant_id)
-        with self.firewall_policy(tenant_id=tenant_id) as fwp:
-            fwp_id = fwp['firewall_policy']['id']
-            attrs = self._get_test_firewall_attrs()
-            attrs['firewall_policy_id'] = fwp_id
-            with self.firewall(firewall_policy_id=fwp_id, tenant_id=tenant_id,
-                               admin_state_up=test_db_firewall.ADMIN_STATE_UP
-                               ) as fw:
-                    # router_ids is not present in the firewall db
-                    # but is added in the get_firewalls override by plugin
-                    fw_list = [fw['firewall']]
-                    f = self.callbacks.get_firewalls_for_tenant_without_rules
-                    res = f(ctx, host='dummy')
-                    for fw in res:
-                        del fw['shared']
-                    self.assertEqual(fw_list, res)
-
 
 class TestFirewallAgentApi(base.BaseTestCase):
     def setUp(self):
@@ -564,7 +536,7 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
                 req = self.new_delete_request('firewalls', fw_id)
                 res = req.get_response(self.ext_api)
                 self.assertEqual(exc.HTTPNoContent.code, res.status_int)
-                self.assertRaises(firewall.FirewallNotFound,
+                self.assertRaises(f_exc.FirewallNotFound,
                                   self.plugin.get_firewall,
                                   ctx, fw_id)
 
@@ -578,7 +550,7 @@ class TestFirewallPluginBase(TestFirewallRouterInsertionBase,
                 req = self.new_delete_request('firewalls', fw_id)
                 res = req.get_response(self.ext_api)
                 self.assertEqual(exc.HTTPNoContent.code, res.status_int)
-                self.assertRaises(firewall.FirewallNotFound,
+                self.assertRaises(f_exc.FirewallNotFound,
                                   self.plugin.get_firewall,
                                   ctx, fw_id)
 
@@ -765,7 +737,7 @@ class TestFirewallRouterPluginBase(test_db_firewall.FirewallPluginDbTestCase,
         fdb.Firewall_db_mixin.\
             supported_extension_aliases = ["fwaas",
                                            "fwaasrouterinsertion"]
-        fdb.Firewall_db_mixin.path_prefix = firewall.FIREWALL_PREFIX
+        fdb.Firewall_db_mixin.path_prefix = fwaas_def.API_PREFIX
 
         super(test_db_firewall.FirewallPluginDbTestCase, self).setUp(
             ext_mgr=ext_mgr,
@@ -777,12 +749,12 @@ class TestFirewallRouterPluginBase(test_db_firewall.FirewallPluginDbTestCase,
             app = config.load_paste_app('extensions_test_app')
             self.ext_api = api_ext.ExtensionMiddleware(app, ext_mgr=ext_mgr)
 
-        self.l3_plugin = directory.get_plugin(nl_constants.L3)
+        self.l3_plugin = directory.get_plugin(plugin_constants.L3)
         self.plugin = directory.get_plugin('FIREWALL')
 
     def test_get_firewall_tenant_ids_on_host_with_associated_router(self):
         agent = helpers.register_l3_agent("host1")
-        tenant_id = str(uuid.uuid4())
+        tenant_id = uuidutils.generate_uuid()
         ctxt = context.get_admin_context()
 
         with self.router(name='router1', admin_state_up=True,
@@ -799,7 +771,7 @@ class TestFirewallRouterPluginBase(test_db_firewall.FirewallPluginDbTestCase,
     def test_get_firewall_tenant_ids_on_host_without_associated_router(self):
         agent1 = helpers.register_l3_agent("host1")
         helpers.register_l3_agent("host2")
-        tenant_id = str(uuid.uuid4())
+        tenant_id = uuidutils.generate_uuid()
         ctxt = context.get_admin_context()
 
         with self.router(name='router1', admin_state_up=True,
@@ -815,8 +787,8 @@ class TestFirewallRouterPluginBase(test_db_firewall.FirewallPluginDbTestCase,
 
     def test_get_firewall_tenant_ids_on_host_with_routers(self):
         agent1 = helpers.register_l3_agent("host1")
-        tenant_id1 = str(uuid.uuid4())
-        tenant_id2 = str(uuid.uuid4())
+        tenant_id1 = uuidutils.generate_uuid()
+        tenant_id2 = uuidutils.generate_uuid()
         ctxt = context.get_admin_context()
 
         with self.router(name='router1', admin_state_up=True,
