@@ -24,6 +24,7 @@ from neutron_lib.plugins import directory
 from oslo_config import cfg
 import six
 
+from neutron_fwaas.db.firewall.v2 import firewall_db_v2
 import neutron_fwaas.extensions
 from neutron_fwaas.extensions import firewall_v2
 from neutron_fwaas.services.firewall import fwaas_plugin_v2
@@ -121,8 +122,7 @@ class TestFirewallRouterPortBase(
 class TestFirewallCallbacks(TestFirewallRouterPortBase):
 
     def setUp(self):
-        super(TestFirewallCallbacks,
-              self).setUp(fw_plugin=FW_PLUGIN_KLASS)
+        super(TestFirewallCallbacks, self).setUp(fw_plugin=FW_PLUGIN_KLASS)
         self.callbacks = self.plugin.endpoints[0]
 
     def test_set_firewall_group_status(self):
@@ -175,7 +175,11 @@ class TestFirewallCallbacks(TestFirewallRouterPortBase):
         def getdelete(context, fwg_id):
             fwg_db = _get_firewall_group(context, fwg_id)
             # NOTE(cby): Use a different session to simulate a concurrent del
-            self.plugin.delete_db_firewall_group_object(alt_ctx, fwg_id)
+            with mock.patch.object(
+                    firewall_db_v2.Firewall_db_mixin_v2,
+                    'delete_firewall_group',
+                    return_value=None):
+                self.plugin.delete_db_firewall_group_object(alt_ctx, fwg_id)
             return fwg_db
 
         with self.firewall_policy() as fwp:
@@ -190,7 +194,6 @@ class TestFirewallCallbacks(TestFirewallRouterPortBase):
                     fwg_db = self.plugin._get_firewall_group(ctx, fwg_id)
                     fwg_db['status'] = nl_constants.PENDING_DELETE
                     ctx.session.flush()
-
                 with mock.patch.object(
                     self.plugin, '_get_firewall_group', side_effect=getdelete
                 ):
@@ -225,11 +228,13 @@ class TestFirewallCallbacks(TestFirewallRouterPortBase):
 
 
 class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
-                             test_l3_plugin.L3NatTestCaseMixin):
+                               test_l3_plugin.L3NatTestCaseMixin):
 
     def setUp(self):
         super(TestFirewallPluginBasev2, self).setUp(fw_plugin=FW_PLUGIN_KLASS)
         fake_notifier.reset()
+        mock.patch.object(self.plugin, '_is_supported_by_fw_l2_driver',
+                          return_value=True).start()
 
     @property
     def _self_context(self):
@@ -670,6 +675,11 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                                           s['subnet']['id'],
                                           None)
 
+    def _get_user_context(self, user_id="a_user", tenant_id="some_tenant",
+                          is_admin=False):
+        return context.Context(user_id=user_id, tenant_id=tenant_id,
+                               is_admin=is_admin)
+
     def test_update_firewall_group_with_invalid_project(self):
         with self.router(name='router1',
                          admin_state_up=True,
@@ -683,8 +693,11 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
             with self.firewall_group(name='test',
                                      default_policy=False,
                                      ports=[],
-                                     admin_state_up=True) as fwg1:
+                                     admin_state_up=True,
+                                     ctx=self._get_user_context()) as fwg1:
                 data = {'firewall_group': {'ports': [port_id]}}
+                # make sure that an exception is raised when admin tries to
+                # update ports of another tenant
                 req = self.new_update_request(
                     'firewall_groups', data, fwg1['firewall_group']['id'],
                     context=context.get_admin_context())
@@ -762,3 +775,68 @@ class TestFirewallPluginBasev2(TestFirewallRouterPortBase,
                         r['router']['id'],
                         s2['subnet']['id'],
                         None)
+
+
+class TestAutomaticAssociation(TestFirewallPluginBasev2):
+
+    def setUp(self):
+        # TODO(yushiro): Replace constant value for this test class
+        # Set auto association fwg
+        super(TestAutomaticAssociation, self).setUp()
+        self.agent_rpc = self.plugin.agent_rpc
+        self.plugin.set_port_for_default_firewall_group = mock.Mock()
+        self.plugin._get_fwg_port_details = mock.Mock()
+
+    def test_vm_port(self):
+        self.plugin._get_fwg_port_details = mock.Mock()
+        kwargs = {
+            "context": mock.ANY,
+            "port": {"id": "fake_port",
+                     "device_owner": "compute:nova",
+                     "binding:vif_type": "ovs",
+                     "project_id": "fake_project"},
+            "original_port": {"binding:vif_type": "unbound"}
+        }
+        self.plugin.handle_update_port(
+            "PORT", "after_update", "test_plugin", **kwargs)
+        self.plugin.set_port_for_default_firewall_group.\
+            assert_called_once_with(mock.ANY,
+                                    kwargs['port']['id'],
+                                    kwargs['port']['project_id'])
+
+    def test_vm_port_not_newly_created(self):
+        # Just updated for VM port(name or description...etc.)
+        kwargs = {
+            "context": mock.ANY,
+            "port": {
+                "id": "fake_port",
+                "device_owner": "compute:nova",
+                "binding:vif_type": "ovs",
+                "project_id": "fake_project"
+            },
+            "original_port": {
+                "device_owner": "compute:nova",
+                "binding:vif_type": "ovs",
+                "project_id": "fake_project"
+            }
+        }
+        self.plugin.handle_update_port(
+            "PORT", "after_update", "test_plugin", **kwargs)
+        self.plugin.set_port_for_default_firewall_group.assert_not_called()
+
+    def test_not_vm_port(self):
+        for device_owner in ["network:router_interface",
+                             "network:router_gateway",
+                             "network:dhcp"]:
+            kwargs = {
+                "context": mock.ANY,
+                "port": {"id": "fake_port",
+                         "device_owner": device_owner,
+                         "project_id": "fake_project"},
+                "original_port": {"device_owner": device_owner,
+                                  "binding:vif_type": "unbound",
+                                  "project_id": "fake_project"}
+            }
+            self.plugin.handle_update_port(
+                "PORT", "after_update", "test_plugin", **kwargs)
+            self.plugin.set_port_for_default_firewall_group.assert_not_called()
